@@ -20,11 +20,8 @@ The plugin handles two cases:
 
 =item A file being present with a name such as /etc/blacklist.d/1.2.3.4
 
-=item A file being present with a name such as $state/blacklist.d/1.2.3.4.
-
 =item An entry in /etc/blogspam/badips matching the incoming IP
 
-
 =back
 
 Note that the server administrator is responsible for populating the
@@ -32,16 +29,19 @@ named directory, or file.
 
 =cut
 
+=head1 LICENSE
+
+This code is licensed under the terms of the GNU General Public
+License, version 2.  See included file GPL-2 for details.
+
+=cut
+
 =head1 AUTHOR
 
-=over 4
-
-=item Steve Kemp
-
+Steve
+--
 http://www.steve.org.uk/
 
-=back
-
 =cut
 
 =head1 LICENSE
@@ -70,6 +70,8 @@ use warnings;
 
 Constructor.  Called when this plugin is instantiated.
 
+This merely saves away the name of our plugin.
+
 =end doc
 
 =cut
@@ -81,6 +83,9 @@ sub new
 
     my $self = {};
 
+    # plugin name
+    $self->{ 'name' } = $proto;
+
     # blacklist file.
     $self->{ 'blacklist-file' } = "/etc/blogspam/badips";
 
@@ -96,6 +101,22 @@ sub new
 
 
 
+=begin doc
+
+Return the name of this plugin.
+
+=end doc
+
+=cut
+
+sub name
+{
+    my ($self) = (@_);
+    return ( $self->{ 'name' } );
+}
+
+
+
 =begin doc
 
 Block a comment if the IP address it has been submitted from has
@@ -139,20 +160,9 @@ sub testComment
     #
     my $dir = $self->{ 'backlist-dir' } || undef;
 
-    if ( defined($dir) && ( -e "$dir/$ip" ) )
+    if ( defined($dir) && ( -e "/etc/blacklist.d/$ip" ) )
     {
-        return "SPAM:badip:$dir/$ip";
-    }
-
-
-    #
-    #  Get our state directory.
-    #
-    my $state = $params{ 'parent' }->getStateDir();
-    my $cdir  = $state . "/blacklist.d/";
-    if ( -d $cdir && -e "$cdir/$ip" )
-    {
-        return "SPAM:badip:state/blacklist.d/$ip";
+        return "SPAM:badip:/etc/blacklist.d/$ip";
     }
 
     #
@@ -182,7 +192,7 @@ sub testComment
         $self->{ 'ips' } = undef;
 
         $self->{ 'verbose' } &&
-          print "re-reading blacklist file $file\n";
+          print $self->name() . ": re-reading blacklist file $file\n";
 
         if ( open( IPS, "<", $file ) )
         {
