@@ -1,55 +1,4 @@
 
-=head1 NAME
-
-Blog::Spam::Plugin::bayasian - Bayasian analysis of submitted comments.
-
-=cut
-
-=head1 ABOUT
-
-This plugin is designed to perform Bayasian analysis upon the content of
-submitted comments, with an aim of automatically rejecting comments which
-have spammy contents.
-
-=cut
-
-=head1 DETAILS
-
-The plugin relies upon a system-wide installation of the B<spambayes>
-script, which is used to analyse contents and judge comment SPAM/HAM
-ratios.
-
-Each site which submits comments against the server will have its own
-unique Spambayes database - so there is no notion of the server storing
-all comment details in a single database.
-
-=cut
-
-=head1 AUTHOR
-
-=over 4
-
-=item Steve Kemp
-
-http://www.steve.org.uk/
-
-=back
-
-=cut
-
-=head1 LICENSE
-
-Copyright (c) 2008-2010 by Steve Kemp.  All rights reserved.
-
-This module is free software;
-you can redistribute it and/or modify it under
-the same terms as Perl itself.
-The LICENSE file contains the full text of the license.
-
-=cut
-
-
-
 package Blog::Spam::Plugin::bayasian;
 
 
@@ -57,7 +6,6 @@ use strict;
 use warnings;
 
 
-use Fcntl qw(:flock);
 use File::Path qw/mkpath/;
 use IPC::Open2;
 
@@ -66,6 +14,8 @@ use IPC::Open2;
 
 Constructor.  Called when this plugin is instantiated.
 
+This merely saves away the name of our plugin.
+
 =end doc
 
 =cut
@@ -76,6 +26,7 @@ sub new
     my $class = ref($proto) || $proto;
 
     my $self = {};
+    $self->{ 'name' } = $proto;
 
     # verbose?
     $self->{ 'verbose' } = $supplied{ 'verbose' } || 0;
@@ -85,6 +36,14 @@ sub new
 }
 
 
+sub name
+{
+    my ($self) = (@_);
+    return ( $self->{ 'name' } );
+}
+
+
+
 =begin doc
 
 This code users the bayasian classification system "spambayes" to test
@@ -125,6 +84,7 @@ sub testComment
     #
     my $db = $dbdir . "/" . $site . ".db";
 
+
     #
     #  If the database doesn't exist we must create it.
     #
@@ -138,12 +98,6 @@ sub testComment
     #
     my $comment = $params{ 'comment' } || "";
 
-    #
-    #  Create a lock
-    #
-    open( FILE, ">", $db . ".lock" );
-    flock( FILE, LOCK_EX );
-
     #
     #  Open the command for reading/writing.
     #
@@ -177,12 +131,6 @@ sub testComment
     #
     waitpid $pid, 0;
 
-    #
-    #  Unlock
-    #
-    flock( FILE, LOCK_UN );
-    close(FILE);
-
     #
     #  We'll not count an "unsure" result, so we're either
     # looking for a header of "ham" or "spam".
@@ -224,6 +172,8 @@ sub classifyComment
     }
 
 
+
+
     #
     #  Get access to our state directory.
     #
@@ -245,16 +195,9 @@ sub classifyComment
     my $db = $dbdir . "/" . $site . ".db";
 
     #
-    #  If the file doesn't exist we must create it.
+    #  If the file doesn't exist we cannot train.
     #
-    #  TODO:
-    #  TODO: Is it an error to train against an empty DB?
-    #  TODO:
-    #
-    if ( !-e $db )
-    {
-        system("sb_filter.py -n -d $db");
-    }
+    return unless ( -e $db );
 
     #
     #  Get the comment body.
@@ -263,42 +206,44 @@ sub classifyComment
     return 0 if ( !defined($body) || !length($body) );
 
 
+    #
+    #  Create the spambayes database if it is missing
+    #
+    if ( !-e $db )
+    {
+        system( "sb_filter.py", "-n", "-d", $db );
+    }
+
+    #
+    #  Construct a logfile to record this against.
+    #
+    my $tmp = "/var/log/blogspam/trained-as-$train.$$." . localtime;
+
     #
     #  Map training to appropriate argument
     #
-    my $flag = "";
     if ( $train =~ /ok/i )
     {
-        $flag = "-g";
+        $train = "-g";
     }
     elsif ( $train =~ /spam/i )
     {
-        $flag = "-s";
+        $train = "-s";
     }
 
     #
     #  Now train
     #
-    open( HANDLE, "|sb_filter.py -d $db $flag -f >/dev/null" );
+    open( HANDLE, "|sb_filter.py -d $db $train -f >/dev/null" );
     print HANDLE $body;
     close HANDLE;
 
     #
     #  Log to see if we have malicious training.
     #
-    my $trained = $state . "/retrained/$train/";
-    mkpath( $trained, { verbose => 0 } ) unless ( -d $trained );
-
-    my $tmp = $trained . time . ".$$";
     if ( open( LOG, ">", $tmp ) )
     {
-        foreach my $key ( keys %params )
-        {
-            next if ( $key =~ /^comment$/i );
-
-            print LOG $key . ": " . $params{ $key } . "\n";
-        }
-        print LOG "\n" . $body . "\n";
+        print LOG $body;
         close(LOG);
     }
 
