@@ -1,49 +1,4 @@
 
-=head1 NAME
-
-Blog::Spam::Plugin::stopwords - Reject comments which contain known-bad words.
-
-=cut
-
-=head1 ABOUT
-
-This plugin is designed to discard comments which contain terms listed
-in a local blacklist.
-
-=cut
-
-=head1 DETAILS
-
-The blacklist must be created and updated by the maintainer of the
-local system, and will be found at B</etc/blogspam/stoplist>.
-
-=cut
-
-=head1 AUTHOR
-
-=over 4
-
-=item Steve Kemp
-
-http://www.steve.org.uk/
-
-=back
-
-=cut
-
-=head1 LICENSE
-
-Copyright (c) 2008-2010 by Steve Kemp.  All rights reserved.
-
-This module is free software;
-you can redistribute it and/or modify it under
-the same terms as Perl itself.
-The LICENSE file contains the full text of the license.
-
-=cut
-
-
-
 
 package Blog::Spam::Plugin::stopwords;
 
@@ -56,6 +11,8 @@ use warnings;
 
 Constructor.  Called when this plugin is instantiated.
 
+This merely saves away the name of our plugin.
+
 =end doc
 
 =cut
@@ -66,18 +23,24 @@ sub new
     my $class = ref($proto) || $proto;
 
     my $self = {};
+    $self->{ 'name' } = $proto;
 
     # verbose?
     $self->{ 'verbose' } = $supplied{ 'verbose' } || 0;
 
-    # the file of stop-words
-    $self->{ 'stopwords' } = "/etc/blogspam/stopwords";
-
     bless( $self, $class );
     return $self;
 }
 
 
+sub name
+{
+    my ($self) = (@_);
+    return ( $self->{ 'name' } );
+}
+
+
+
 =begin doc
 
 Block if we find some particular stop-words in the body of the message.
@@ -111,13 +74,7 @@ sub testComment
     #
     #  The stop-words come from this file.
     #
-    my $file = $self->{ 'stopwords' } || undef;
-
-    #
-    #  No file?  Then return.
-    #
-    return "OK" if ( !defined($file) || !-e $file );
-
+    my $file = "/etc/blogspam/stopwords";
 
     #
     #  Get the mtime of the file.
@@ -137,33 +94,13 @@ sub testComment
 
         if ( open( WORDS, "<", $file ) )
         {
-
-            $self->{ 'words' } = undef;
-            $self->{ 'verbose' } &&
-              print "re-reading stopwords from $file\n";
-
-            while ( my $line = <WORDS> )
+            while (<WORDS>)
             {
+                my $word = $_;
+                chomp($word);
+                next if ( !length($word) );
 
-                # skip blank lines
-                next unless ( $line && length($line) );
-
-                # Skip lines beginning with comments
-                next if ( $line =~ /^([ \t]*)\#/ );
-
-                # strip spaces
-                $line =~ s/^\s+|\s+$//g;
-
-                # strip newline
-                chomp($line);
-
-                # empty now?
-                next unless length($line);
-
-                $self->{ 'words' }{ lc $line } = 1;
-
-                $self->{ 'verbose' } && print "Blacklisting term: $line\n";
-
+                $self->{ 'words' }{ $word } = 1;
             }
             close(WORDS);
             $self->{ 'wmtime' } = $mtime;
@@ -186,7 +123,7 @@ sub testComment
 
         if ( $self->{ 'words' }{ $word } )
         {
-            return "SPAM:stopwords:$word";
+            return "SPAM:$word";
         }
     }
 
