@@ -1,48 +1,4 @@
 
-=head1 NAME
-
-Blog::Spam::Plugin::surbl - Discard comments with surbl-listed URLs.
-
-=cut
-
-=head1 ABOUT
-
-This plugin is designed to discard comments which contain URLS which
-are listed in the surbl database.
-
-For more details of the surbl service please consult:
-
-=over 8
-
-=item http://www.surbl.org/
-
-=back
-
-=cut
-
-=head1 AUTHOR
-
-=over 4
-
-=item Steve Kemp
-
-http://www.steve.org.uk/
-
-=back
-
-=cut
-
-=head1 LICENSE
-
-Copyright (c) 2008-2010 by Steve Kemp.  All rights reserved.
-
-This module is free software;
-you can redistribute it and/or modify it under
-the same terms as Perl itself.
-The LICENSE file contains the full text of the license.
-
-=cut
-
 
 package Blog::Spam::Plugin::surbl;
 
@@ -57,6 +13,8 @@ use Net::DNS::Resolver;
 
 Constructor.  Called when this plugin is instantiated.
 
+This merely saves away the name of our plugin.
+
 =end doc
 
 =cut
@@ -67,6 +25,7 @@ sub new
     my $class = ref($proto) || $proto;
 
     my $self = {};
+    $self->{ 'name' } = $proto;
 
     # verbose?
     $self->{ 'verbose' } = $supplied{ 'verbose' } || 0;
@@ -76,6 +35,13 @@ sub new
 }
 
 
+sub name
+{
+    my ($self) = (@_);
+    return ( $self->{ 'name' } );
+}
+
+
 
 =begin doc
 
