@@ -8,33 +8,125 @@ Blog::Spam::Server - An RPC server which detects comment spam.
 =head1 ABOUT
 
 This program implements a plugin-based XML-RPC server which may be
-queried from almost all languages.
+queried from a number of languages.
 
 The intention is that clients will query this server to detect whether
-their comment submissions are spam or genuine, using the API we present
-over the network via XML::RPC - that API is documented in L<Blog::Spam::API>.
+their comment submissions are spam or genuine.
 
-The actual testing of submitted comments is handled by a series of
-plugins, each living beneath the 'Blog::Spam::Plugin::' namespace.
+Whilst it isn't likely to be amazingly comprehensive it should do
+better than simple tests are able to do locally, and the benefit
+of running your own server is that you can centralise the testing which
+is used by multiple web applications.
 
-For a description of the plugin methods please consult the
-L<sample plugin|Blog::Spam::Plugin::Sample>.
+(Of course if you trust an external third party then you have a new
+single point of failure - thats why the server is open-source.  Run
+your own!)
 
 =cut
 
+=head1 PLUGINS
+
+The core of the server is a collection of plugins, each of which
+will be loaded dynamically from the Blog::Spam::Plugin namespace.
+
+Each plugin will be passed a hash of data from the client, and it
+is expected that the plugin will return a single string result:
+
+   "ok"   =>  The comment is OK, and the next plugin should be called.
+
+   "spam" =>  The comment is spam, and further plugins should be skipped.
+
+   "good" =>  The comment is good, and all further plugins should be skipped.
+
+=cut
+
+
+=head1 CALLING
+
+To call the XML-RPC server you should pass the following parameters
+in a struct:
+
+   agent   -> The user-agent of the submitting browser, if any.
+   comment -> The body of the comment
+   email   -> The email address submitted
+   ip      -> The IP the comment was submitted from.
+   name    -> The name the user chose, if any.
+
+   options -> Discussed later.
+
+The only mandatory arguments are "comment" and "IP", the rest are
+optional but may be useful and it is recommended you pass them if
+you can.
+
+The return value will either be "OK", or "SPAM: with a reason".
+
+=cut
+
+
+=head1 OPTIONS
+
+You may pass the optional "options" string to the server, if you
+wish finer control.
+
+This option string should consist of comma-separated tokens.
+
+The permissible values are:
+
+         whitelist=1.2.3.4      -> Whitelist the given IP / CIDR range.
+         blacklist=1.2.3.4      -> Blacklist the given IP / CIDR range.
+
+         exclude=plugin         -> Don't run the plugin with name "plugin".
+
+         mandatory=subject      -> Specify the given field should always be
+                                   present.
+
+         max-links=20           -> The maximum number of URLs.
+
+         min-size=1024          -> Minimum body size.
+
+         max-size=2k            -> Maximum body size
+
+         fail                   -> Always return "spam"
+
+These options may be repeated, for example the following is a valid
+value for the "options" setting:
+
+=for example begin
+
+   mandatory=subject,mandatory=name,whitelist=1.2.3.4,exclude=surbl
+
+=for example end
+
+That example will:
+
+1.  Make the "subject" field mandatory.
+
+2.  Makes the "name" field mandatory.
+
+3.  Whitelists any comments subitted from the IP 1.2.3.4
+
+4.  Causes the server to not run the surbl test.
+
+=cut
+
+=cut
+
+=head1 LICENSE
+
+This code is licensed under the terms of the GNU General Public
+License, version 2.  See included file GPL-2 for details.
+
+=cut
+
+
 =head1 AUTHOR
 
-=over 4
-
-=item Steve Kemp
-
+Steve
+--
 http://www.steve.org.uk/
 
-=back
-
 =cut
 
-
 =head1 LICENSE
 
 Copyright (c) 2008-2010 by Steve Kemp.  All rights reserved.
@@ -51,7 +143,7 @@ package Blog::Spam::Server;
 
 
 use vars qw($VERSION);
-our $VERSION = "0.9.2";
+our $VERSION = "0.1";
 
 #
 #  The modules we require
@@ -81,11 +173,9 @@ use warnings;
 
 
 
-=begin doc
+=head2 new
 
-Create a new instance of this object.
-
-=end doc
+  Create a new instance of this object.
 
 =cut
 
@@ -104,6 +194,11 @@ sub new
         $self->{ lc $key } = $supplied{ $key };
     }
 
+    #
+    #  Spam ID.
+    #
+    $self->{ 'id' } = 0;
+
     #
     #  Open syslog
     #
@@ -121,7 +216,7 @@ sub new
 
 =begin doc
 
-Create our child XML-RPC server.
+  Create the XML-RPC server.
 
 =end doc
 
@@ -135,24 +230,11 @@ sub createServer
     $options{ 'port' } = $params{ 'port' } || 8888;
     $options{ 'host' } = $params{ 'host' } if ( $params{ 'host' } );
 
-    $self->{ 'verbose' } && print "Creating RPC server.\n";
-
     #
     # Create the server object.
     #
     $self->{ 'daemon' } = RPC::XML::Server->new(%options);
 
-    #
-    # Did we fail to bind?
-    #
-    if ( ( !$self->{ 'daemon' } ) ||
-         ( !$self->{ 'daemon' }->can("add_method") ) )
-    {
-        print "ERROR: Failed to bind!\n";
-        return undef;
-    }
-
-    $self->{ 'verbose' } && print "Adding RPC methods.\n";
 
     #
     # Add our 'testComment' method.
@@ -190,132 +272,29 @@ sub createServer
           code      => sub {$self->getPlugins(@_)}
         } );
 
-}
+    $self->{ 'verbose' } && print "Added RPC methods\n";
 
 
-
-=begin doc
-
-Load the plugins we might have present.
-
-The return value to this method will be the number of plugins which
-were successfully loaded.
-
-=end doc
-
-=cut
-
-sub loadPlugins
-{
-    my ($self) = (@_);
-
-    $self->{ 'verbose' } && print "Loading plugins.\n";
-
-    my $count = 0;
-
     #
-    #  Load our plugins, passing allong the verbosity
-    # value.
+    #  Load our plugins once.
     #
     my @plugins = $self->plugins( verbose => $self->{ 'verbose' } );
 
-    #
-    #  Ignore plugins that don't implement "name()".
-    #
-    my @valid;
-
-    foreach my $plugin (@plugins)
-    {
-
-        #
-        #  Make sure the exported method is there.
-        #
-        push( @valid, $plugin );
-        $count += 1;
-    }
-
-
     #
     #  Sort by name.
     #
-    if ($count)
-    {
-        my @sorted = map {$_->[0]}
-          sort {$a->[1] cmp $b->[1]}
-          map {[$_, ( ref $_ )]} @valid;
+    my @sorted = map {$_->[0]}
+      sort {$b->[1] cmp $a->[1]}
+      map {[$_, ( $_->name() )]} @plugins;
 
-        #
-        #  Squirrel them away for invokation later.
-        #
-        @{ $self->{ 'plugins' } } = @sorted;
-    }
-    else
-    {
-        print
-          "ERROR: Failed to load any plugins from Blog::Spam::Plugin namespace\n";
-    }
-
-
-    #
-    #  Return the count.
-    #
-    $self->{ 'verbose' } && print "Loaded $count plugins\n";
-    return ($count);
+    @{ $self->{ 'plugins' } } = @sorted;
 }
 
 
 
 =begin doc
 
-Run any scheduled tasks.
-
-It is assumed this method will be called from cron.  Something like:
-
-=for example begin
-
-  my $server = Blog::Spam::Server->new()
-  $server->loadPlugins();
-  $server->runTasks( "daily" );
-  exit 0;
-
-=for example end
-
-=end doc
-
-=cut
-
-sub runTasks
-{
-    my ( $self, $label ) = (@_);
-
-    $self->{ 'verbose' } && print "Running tasks: $label\n";
-
-    foreach my $plugin ( @{ $self->{ 'plugins' } } )
-    {
-
-        #
-        #  Ignore plugins which don't implement an 'expire' method.
-        #
-        my $name = ref $plugin;
-        next unless ( $plugin->can("expire") );
-
-        #
-        #  Call plugin.
-        #
-        $self->{ 'verbose' } && print "$name - starting\n";
-        $plugin->expire( $self, $label );
-        $self->{ 'verbose' } && print "$name - done\n";
-    }
-
-    $self->{ 'verbose' } && print "All registered scheduler tasks executed.\n";
-}
-
-
-
-
-=begin doc
-
-Run the main loop of our L<RPC::XML::Server> object and don't return.
+  Run the main loop and don't return.
 
 =end doc
 
@@ -325,15 +304,6 @@ sub runLoop
 {
     my ($self) = (@_);
 
-    $self->{ 'verbose' } && print "Starting server loop\n";
-
-    if ( ( !$self->{ 'daemon' } ) ||
-         ( !$self->{ 'daemon' }->can("add_method") ) )
-    {
-        print "ERROR: Server not loaded!\n";
-        return undef;
-    }
-
     $self->{ 'daemon' }->server_loop();
 
 }
@@ -343,7 +313,7 @@ sub runLoop
 
 =begin doc
 
-This method is invoked for each incoming SPAM test.
+  This method is invoked for each incoming blog spam test.
 
 =end doc
 
@@ -357,19 +327,7 @@ sub testComment
     #  The parameters the user submitted, as a hash so
     # they're easy to work with.
     #
-    my %struct;
-
-
-    #
-    #  Copy each supplied value to the struct we'll use
-    # from here on in - but lower-case the key-names.
-    #
-    foreach my $key ( keys %$struct )
-    {
-        my $lkey = lc($key);
-
-        $struct{ $lkey } = $struct->{ $key };
-    }
+    my %struct = %$struct;
 
     #
     #  Log the peer.
@@ -381,27 +339,9 @@ sub testComment
         $self->{ 'verbose' } &&
           print "Connection from " . $struct{ 'peer' } . "\n";
 
+
     }
 
-
-
-    #
-    #  If we've got an IPv6 prefix remove it.
-    #
-    if ( $struct{ 'ip' } && ( $struct{ 'ip' } =~ /^::ffff:(.*)/i ) )
-    {
-        $struct{ 'ip' } = $1;
-    }
-
-
-    #
-    #  Pass ourself over to the plugin - primarily so that
-    # a plugin may call L<getState> to find a location to
-    # persist data to.
-    #
-    $struct{ 'parent' } = $self;
-
-
     #
     #  The mandatory values we expect by default.
     #
@@ -426,10 +366,7 @@ sub testComment
     #
     foreach my $option ( split( /,/, $options ) )
     {
-        $option =~ s/^\s+|\s+$//g;
-        next unless ( length($option) );
-
-        if ( $option =~ /^mandatory=(.*)/i )
+        if ( $option =~ /mandatory=(.*)/i )
         {
             my $key = $1;
 
@@ -439,7 +376,7 @@ sub testComment
                 return "SPAM:Missing $key";
             }
         }
-        if ( $option =~ /^exclude=(.*)/i )
+        if ( $option =~ /exclude=(.*)/i )
         {
             $skip{ $1 } = 1;
         }
@@ -460,15 +397,10 @@ sub testComment
     foreach my $plugin ( @{ $self->{ 'plugins' } } )
     {
 
-        #
-        #  Ignore plugins that don't implement testComment.
-        #
-        next unless ( $plugin->can("testComment") );
-
         #
         #  The name of the plugin
         #
-        my $name = ref $plugin;
+        my $name = $plugin->name();
 
 
         my $skipThis = 0;
@@ -504,17 +436,12 @@ sub testComment
             }
 
         }
-
-        #
-        #  Skip all further tests if either:
-        #
-        #   1.  We already ahve a result.
-        #
-        #   2.  This particular plugin is to be excluded, as
-        #      decided by the caller.
-        #
         next if ($skipThis);
 
+        #
+        #  Pass ourself over to the plugin
+        #
+        $struct{ 'parent' } = $self;
 
         #
         #  Call the plugin
@@ -542,7 +469,7 @@ sub testComment
         #
         if ( $result =~ /^spam:/i )
         {
-            $blocker = ref $plugin;
+            $blocker = $plugin->name();
             if ( $blocker =~ /(.*)::(.*)/ )
             {
                 $blocker = $2;
@@ -570,43 +497,12 @@ sub testComment
         $struct{ 'result' } = $result
           if ( defined($result) && length($result) );
 
-
-        #
-        #  Bump the statistics for the appropriate domain.
-        #
-        $self->bumpStats(%struct);
-
-        #
-        #  See if any plugin will handle the logging too.
-        #
-        foreach my $plugin ( @{ $self->{ 'plugins' } } )
-        {
-
-            #
-            #  Ignore plugins that don't implement logMessage
-            #
-            next unless ( $plugin->can("logMessage") );
-
-            #
-            #  Call any loggin plugins
-            #
-            $self->{ 'verbose' } &&
-              print "\tLogging result with " . ref $plugin . "\n";
-            $plugin->logMessage( $self, %struct );
-        }
+        $self->logMessage(%struct);
     }
 
 
     #
-    #  OK so if we weren't testing we'll have logged
-    # the message, the result, and the blocker to
-    # a file which will get rotated out.
-    #
-    #  If we're running with --verbose we should
-    # now log a summary to the console.
-    #
-    #  Either way, verbose or not, we will log a summary
-    # to syslog.
+    #  Show the result to the console
     #
     my $time = "[" . localtime(time) . "] ";
 
@@ -614,23 +510,15 @@ sub testComment
     # Build up a message.
     #
     my $msg = "";
-    $msg .= "TEST "                 if ( $struct{ 'test' } );
-    $msg .= "PEER:$struct{'peer'} ";
-    $msg .= "SITE:$struct{'site'} " if ( $struct{ 'site' } );
-    $msg .= "IP:$struct{'ip'} ";
-    $msg .= " BLOCKER:" . $blocker  if ($blocker);
-    $msg .= " RESULT:" . $result    if ($result);
+    $msg .= "TEST "                if ( $struct{ 'test' } );
+    $msg .= "IP:$struct{'ip'}";
+    $msg .= " BLOCKER:" . $blocker if ($blocker);
+    $msg .= " RESULT:" . $result   if ($result);
 
     #
     #  Show to console if we're verbose.
     #
-    print "\t$time $msg\n" if ( $self->{ 'verbose' } );
-
-    #
-    #  But always log to syslog
-    #
-    syslog( "info|local0", $msg );
-
+    $self->{ 'verbose' } && print "\t$time $msg\n";
 
     #
     # We remap "good" to be "OK" so that we need only document
@@ -647,111 +535,6 @@ sub testComment
 }
 
 
-
-
-=begin doc
-
-This method is invoked when a comment is to be re-trained.
-
-=end doc
-
-=cut
-
-sub classifyComment
-{
-    my ( $self, $xmlrpc, $struct ) = (@_);
-
-    #
-    #  The parameters the user submitted, as a hash so
-    # they're easy to work with.
-    #
-    my %struct;
-
-
-    #
-    #  Copy each supplied value to the struct we'll use
-    # from here on in - but lower-case the key-names.
-    #
-    foreach my $key ( keys %$struct )
-    {
-        my $lkey = lc($key);
-
-        $struct{ $lkey } = $struct->{ $key };
-    }
-
-    #
-    #  Log the peer.
-    #
-    if ( $xmlrpc->{ 'peerhost' } )
-    {
-        $struct{ 'peer' } = $xmlrpc->{ 'peerhost' };
-
-        $self->{ 'verbose' } &&
-          print "classifyComment: connection from " . $struct{ 'peer' } . "\n";
-
-    }
-
-
-
-    #
-    #  If we've got an IPv6 prefix remove it.
-    #
-    if ( $struct{ 'ip' } && ( $struct{ 'ip' } =~ /^::ffff:(.*)/i ) )
-    {
-        $struct{ 'ip' } = $1;
-    }
-
-
-    #
-    #  Pass ourself over to the plugin - primarily so that
-    # a plugin may call L<getState> to find a location to
-    # persist data to.
-    #
-    $struct{ 'parent' } = $self;
-
-
-    #
-    #  Call each plugin, which implements the classifycomment method,
-    # in sorted order.
-    #
-    foreach my $plugin ( @{ $self->{ 'plugins' } } )
-    {
-
-        #
-        #  Ignore plugins that don't implement testComment.
-        #
-        next unless ( $plugin->can("classifyComment") );
-
-        #
-        #  The name of the plugin
-        #
-        my $name = ref $plugin;
-
-        #
-        #  Call the plugin
-        #
-        $self->{ 'verbose' } && print "Calling plugin: $name\n";
-
-        my $result = $plugin->classifyComment(%struct);
-
-        if ( defined($result) && length($result) )
-        {
-            $self->{ 'verbose' } && print "\t=> $result\n";
-        }
-        else
-        {
-            $self->{ 'verbose' } && print "\t=> No result returned.\n";
-        }
-    }
-
-    #
-    # Return the result to the caller.
-    #
-    return ("TRAINED");
-
-}
-
-
 =begin doc
 
 This method will return the SPAM/OK counts either globally or for the
@@ -828,7 +611,7 @@ sub getPlugins
 
     foreach my $plugin ( sort $self->plugins() )
     {
-        my $name = ref $plugin || $plugin;
+        my $name = $plugin->name() || $plugin;
 
         if ( $name =~ /::([^:]+)$/ )
         {
@@ -843,13 +626,13 @@ sub getPlugins
 
 =begin doc
 
-Increase the global and per-domain SPAM/OK stats.
+  Log the single message located in the structure to disk.
 
 =end doc
 
 =cut
 
-sub bumpStats
+sub logMessage
 {
     my ( $self, %struct ) = (@_);
 
@@ -900,6 +683,44 @@ sub bumpStats
         $self->increaseCount( $dir . "/$site/count" );
     }
 
+
+    #
+    #  OK so at this point we've increased the SPAM/OK count
+    # both globally and for the specific site.
+    #
+    #  Now we want to log the message itself.
+    #
+
+    #
+    #  The directory we write to.
+    #
+    my $dir = $state . "/" . "logs/";
+    mkpath( $dir, { verbose => 0 } ) unless ( -d $dir );
+
+    #
+    #  Make sure the filename doesn't have any bogus characters
+    # in it.
+    #
+    my $file = $struct{ 'ip' } . "." . $self->{ 'id' } . ".$$";
+    $file =~ s/[^a-zA-Z0-9]/_/g;
+    $file = $dir . $file;
+
+    #
+    #  Bump our ID
+    #
+    $self->{ 'id' } += 1;
+
+    open( LOG, ">", $file );
+    foreach my $key ( sort keys %struct )
+    {
+        if ( $key !~ /^(comment|parent)$/i )
+        {
+            print LOG "$key: " . $struct{ $key } . "\n";
+        }
+    }
+    print LOG "\n";
+    print LOG $struct{ 'comment' } . "\n";
+    close(LOG);
 }
 
 
@@ -998,7 +819,7 @@ sub getStateDir
     my @user = getpwnam("s-blogspam");
     if (@user)
     {
-        return ( $user[7] . "/state/" );
+        return ( $user[7] . "/" );
     }
     else
     {
